/*
Script: colorcircle.js

Copyright:
	Copyright (c) 2008 [Malte Wittkugel](http://wittkugel.net/)

Code & Documentation:
	(http://colorcircle.wittkugel.net/)
*/
var Colorcircle = new Class({
  Implements: [Events, Options],
  
  options: {
    color: "#f00",
    onChange: null,
    onInitialize: null,
    directory: "colorcircle/",
    size: 200
  },

  initialize: function(container, options) {
    if($H(container).$family.name != "element" && typeof(container) != "string") {
      var o = options;
      options = container;
      container = o;
    }
    this.container = $(container);
    if(this.container == null) {
      this.container = new Element("div");
      $(document.body).adopt(this.container);
    }
    
    this.setOptions(options);
    
    this.color = new Color(this.options.color);
    
    if(this.options.directory.substr(this.options.directory.length-2) != "/")
      this.directory += "/";

    //add Colorcircle-Elements
    this.addElements();
    
    this.calculatePosition();
    this.setColor(this.color);
    
    //add Event Handler
    this.eventDiv.addEvent("mousedown", this.mouseDownHandler.bind(this));
  },
  
  addElements: function() {
    this.container.set("styles", {
      "position": "relative",
      "width": this.options.size,
      "height": this.options.size
    });

    this.circle = new Element("div", {
      "styles": {
        "position": "absolute",
        "left": 0,
        "top": 0,
        "width": this.options.size,
        "height": this.options.size,
        "background": "url("+this.options.directory+"circle.png) no-repeat"
      }
    });
    
    this.mask = new Element("div", {
      "styles": {
        "position": "absolute",
        "left": this.options.size / 4,
        "top": this.options.size / 4,
        "width": this.options.size / 2,
        "height": this.options.size / 2,
        "background": this.color.setSaturation(100).setBrightness(100).hex + " url("+this.options.directory+"mask.png) no-repeat"
      }
    }); 
    
    this.circleCursor = new Element("div", {
      "styles": {
        "position": "absolute",
        "width": 14,
        "height": 14,
        "background": "url("+this.options.directory+"cursor.png) no-repeat"
      }
    });
    
    this.maskCursor = this.circleCursor.clone();
    
    this.eventDiv = new Element("div", {
      "styles": {
        "position": "absolute",
        "left": 0,
        "top": 0,
        "width": this.circle.getStyle("width").toInt(),
        "height": this.circle.getStyle("height").toInt(),
        "cursor": "crosshair",
        "MozUserSelect": "none"
      }
    });

    this.container.adopt(this.circle);
    this.container.adopt(this.mask);
    this.container.adopt(this.circleCursor);
    this.container.adopt(this.maskCursor);
    this.container.adopt(this.eventDiv);
  },
  
  calculatePosition: function() {
    this.dimensions = {
      x: this.circle.getPosition().x,
      y: this.circle.getPosition().y,
      radius: this.circle.getSize().x / 16 * 7
    };
  },
  
  mouseDownHandler: function(event) {
    var pos = this.getCoords(event);
    this.dragMask = Math.max(Math.abs(pos.x), Math.abs(pos.y)) <= this.mask.getSize().x / 2;
    this.mouseMoveHandler(event);
    $(document).addEvent("mouseup", this.mouseUpHandler.bind(this));
    $(document).addEvent("mousemove", this.mouseMoveHandler.bind(this));
  },
  
  mouseUpHandler: function(event) {
    $(document).removeEvents("mouseup");
    $(document).removeEvents("mousemove");
  },
  
  mouseMoveHandler: function(event) {
    var pos = this.getCoords(event);
    if(this.dragMask) {
      this.saturation = 100 - Math.round(Math.max(0, Math.min(1, -(pos.x / this.mask.getSize().x) + .5)) * 100);
      this.brightness = 100 - Math.round(Math.max(0, Math.min(1, -(pos.y / this.mask.getSize().x) + .5)) * 100);
      this.color = this.color.setSaturation(this.saturation);
      this.color = this.color.setBrightness(this.brightness);
    } else {
      this.hue = Math.atan2(pos.x, pos.y) / (Math.PI * 2);
      if (this.hue < 0)
        this.hue += 1;
      this.hue = Math.round(this.hue * 360);
      this.color = this.color.setHue(this.hue);
      this.mask.setStyle("backgroundColor", this.color.setSaturation(100).setBrightness(100).hex);
    }
    
    this.updateCursors(event);
    this.fireEvent("onChange", [this.color]);
  },
  
  getCoords: function(event) {
    return {
      x: event.page.x - (this.dimensions.x + this.options.size / 2),
      y: (this.dimensions.y + this.options.size / 2) - event.page.y
    };
  },
  
  updateCursors: function(event) {
    if(this.dragMask) {
      this.maskCursor.set("styles", {
        "left": ((event.page.x < this.mask.getPosition().x) ? this.mask.getPosition().x : (event.page.x > this.mask.getPosition().x + this.mask.getSize().x) ? this.mask.getPosition().x + this.mask.getSize().x : event.page.x) - this.dimensions.x - this.maskCursor.getSize().x / 2,
        "top": ((event.page.y < this.mask.getPosition().y) ? this.mask.getPosition().y : (event.page.y > this.mask.getPosition().y + this.mask.getSize().y) ? this.mask.getPosition().y + this.mask.getSize().y : event.page.y) - this.dimensions.y - this.maskCursor.getSize().x / 2
      });
    } else {
      this.circleCursor.set("styles", {
        "left": Math.round(Math.sin((this.hue * (Math.PI * 2) / 360)) * this.dimensions.radius + this.options.size / 2 - this.maskCursor.getSize().x / 2),
        "top": Math.round(-Math.cos((this.hue * (Math.PI * 2) / 360)) * this.dimensions.radius + this.options.size / 2 - this.maskCursor.getSize().x / 2)
      });
    }
  },
  
  setColor: function(color) {
    this.color = new Color(color);
    this.circleCursor.set("styles", {
      "left": Math.round(Math.sin((this.color.hsb[0] * (Math.PI * 2) / 360)) * this.dimensions.radius + this.options.size / 2 - this.maskCursor.getSize().x / 2),
      "top": Math.round(-Math.cos((this.color.hsb[0] * (Math.PI * 2) / 360)) * this.dimensions.radius + this.options.size / 2 - this.maskCursor.getSize().x / 2)
    });
    this.maskCursor.set("styles", {
      "left": this.color.hsb[1] + this.options.size / 4 - this.maskCursor.getSize().x / 2,
      "top": this.options.size / 4  + this.options.size / 2 - this.color.hsb[2]  - this.maskCursor.getSize().x / 2
    });
    this.mask.setStyle("backgroundColor", this.color.setSaturation(100).setBrightness(100).hex);
    this.fireEvent("onChange", [this.color]);
  }
});
